#' Generate Random Numbers Using Balanced Randomization
#'
#' The function generates random numbers for treatment assignment
#' using balanced randomization.
#'
#' @param q.vec     Vector of randomization weights, one weight for each arm.
#' @param Accrual   The number of patients already enrolled to each arm.
#' @param N.target  Vector of the targeted sample sizes for each arm.
#' @param Active    Activity status, one if active and zero else,
#'                  for each arm at the i-th arrival.
#' @param nr.a      How many random numbers should be generated
#'
#' @details
#' The function generates random numbers \eqn{C_i} with probability proportional to
#' \deqn{p(C_i=a) \propto q_a I(N_a < n_{T,a},}
#' where \eqn{q_a, N_a, n_{T,a}} are
#' the randomization weight,
#' number of patients enrolled and maximal enrollment to arm a.
#'
#' @return
#' Returns a set of \code{nr.a} random treatment assignment numbers.
#'
#' @examples
#' ## 5 arms
#' Accrual  = c(51, 49, 47, 10, 10)   # current accrual
#' N.target = rep(50, 5)              # maximum accrual
#' q.vec    = c(2,1,1,1.5,1.5)        # randomization weights
#' Active   = c(1,0,1,1,0)            # arms 2 and 5 are inactive
#'
#' Randomize.BR.add(q.vec, Accrual, N.target, nr.a=5)
#'
#' @export
Randomize.BR.add =

  function(q.vec, Accrual, N.target, Active=rep(1, length(Accrual)),  nr.a=1)

sample.int(n=length(q.vec), size=nr.a, replace=TRUE, prob=q.vec*(Accrual<N.target)*Active )







#' Simulate a Trial using Balanced Randomization
#'
#' Generate a multi-arm trial using a balanced design,
#' with early stopping for futility,
#' and the possibility to  add arms to the trial.
#'
#' @param Potential.Outcome
#' A matrix with  potential outcome data
#' be generated with \code{\link{Generat.Outcome}}.
#'
#' @param added.at
#' A vector of adding times for each arm.
#' For all initial arms use \code{0}.
#'
#' @param Outcome.delay
#' Expected time between enrolments and avalability of the outcome.
#' (If the endpoint is response after 6 month of treatment then \code{Outcome.delay=6}).
#'
#' @param N.target
#' Vector of the targeted sample size for each arm.
#' No more than \code{N.target[k]} patients will be enrolled to arm k.
#'
#' @param q.vec
#' Vector of randomization weights, one weight for each arm.
#'
#' @param use.concurrent
#' If TRUE only  concurrent control data are used for treatment comparision.
#'
#' @param stop.par           Vector of stopping parameters with default c(f=0, g=1).
#' @param do.futi.stop       If \code{TRUE} do sequantial futility analysis.
#'
#' @return
#' The function returns a list \code{list(Potential.Outcome, Outcome=Outcome, Active=Active)} of three elements.
#'
#' \code{Potential.Outcome} is identical to the impute matrix \code{Potential.Outcome}
#' except that the 2nd  column now contains treatment assignment variables, i.e.
#' \code{Potential.Outcome[10,2]} equals one if the 10th patients was Accrual to the control.
#' \code{Potential.Outcome[10,2]} equals 2 if the 10th patients was Accrual to the arm 1.
#'
#' \code{Outcome} is an array of outcome data,
#' where \code{Outcome[,,k]} corresponds to the outcome data in the kth group of added arms.
#'
#' \code{Active} corresponds to the activity status of each arm.
#'
#' @examples
#' # generate outcome data
#' X = Generat.Outcome(N=5*63, p.vec=rep(.3, 5), A.rate=6)
#'
#' #generate trial
#' Simulate.BR.add.trial(
#'    Potential.Outcome= X,
#'
#'    # add two arms after the enrolment of 74 patients
#'    added.at         = c(rep(0, 3),75, 75),
#'
#'    # at most 50 patients for each arm
#'    N.target         = rep(50, 5),
#'
#'    # equal randomization weithgs
#'    q.vec            = rep(1,5),
#'    stop.par         = c(f=0.3, gf=1))
#'
#' @export
Simulate.BR.add.trial =
  function(Potential.Outcome, added.at=rep(0, ncol(Potential.Outcome)),Outcome.delay=0,  N.target,
           q.vec, stop.par=c(f=0, gf=1), use.concurrent=FALSE, do.futi.stop=FALSE){

    N.A            = dim(Potential.Outcome) - c(0,2)
    M.k            = unique(added.at)
    K              = length(M.k)
    batch          = sapply(added.at, function(a) which(M.k==a))
    Outcome        = array(0L, dim=c(2, N.A[2], K) )
    Accrual     = rep(0, N.A[2])
    n.last         = 0
    N.target.0     = N.target[1]
    Active         = rbind(rep(1L,N.A[2]), rep(0L,N.A[2]))

##############  active accrual period #####################
    for(i in 1:N.A[1]){

      if(any(M.k==i) & use.concurrent)  N.target[1]   = N.target.0 + Accrual[1]

      ## update data during the active accrual period
      id         = added.at<=i
      t.i            = Potential.Outcome[i,1]
      Update.Outcome = Update.Data(i, t.i, Potential.Outcome, Outcome.delay, Outcome, n.last, batch, N.target.0, M.k, during.accrual=TRUE)
      Outcome        = Update.Outcome$Outcome
      n.last         = Update.Outcome$n.last

      ## futility testing
      if(do.futi.stop){
       Outcome.c  = sapply(1:sum(id), function(a) Outcome[,a, batch[a]] )
       PrPTE      = Posterior.with.control(Outcome=Outcome.c)
       Active[,id]= Apply.stopping.rules(i=i, Active=Active[,id], PrPTE=PrPTE, stop.par=stop.par, Accrual=Accrual[id], n.a.max=N.target[id][-1])
      }

      ## randomize
      if( any(Active[1,id][-1]==1 & Accrual[id][-1] <N.target[id][-1])  ){
        Potential.Outcome[i,2] = a.i = Randomize.BR.add(q.vec=q.vec[id], Accrual=Accrual[id], N.target=N.target[id], Active = Active[1,id])
        Accrual[a.i]        = Accrual[a.i] + 1
      }
    }

############## follow up: update data after  the active accrual period #####################
    if(n.last< N.A[1]){
      id                   = rep(TRUE, N.A[2])

      for( j in (n.last+1):N.A[1]){
        if(any(Active[1,id][-1]==1)){
                               a                    = Potential.Outcome[j,2]
          if(a>1)              Outcome[,a,batch[a]] = Outcome[,a,batch[a]] + c(1L, Potential.Outcome[j,2+a])
          if(a==1){
            for(k in 1:sum(M.k <= i)) if(j>M.k[k] & Outcome[1,1,k]<N.target.0)
                               Outcome[,1,k]        = Outcome[,1,k]        + c(1L, Potential.Outcome[j,3])  }

        # apply futility stopping rules
        if(do.futi.stop){
          Outcome.c = sapply(1:sum(id), function(a) Outcome[,a, batch[a]])
          PrPTE     = Posterior.with.control(Outcome=Outcome.c)
          Active    = Apply.stopping.rules(i=i, Active=Active[,id], PrPTE=PrPTE, stop.par=stop.par, Accrual=Accrual[id], n.a.max=N.target[id][-1])
        }

        }}}

    return(list(TrialData=Potential.Outcome, Outcome=Outcome, Active=Active)) }
