#' Generate random numbers for a Bayesian Adaptive treatment assignment
#'
#' Generate a random vector of \code{nr.a} treatment assignment variables for the Bayesian adaptive design.
#'
#' The function generates integers between 1 and A, where A is the number of arms in the trial.
#' Each random integer \eqn{ C_i= \{1,\ldots, A\}} denotes the assignment of patient i to
#' the control (\eqn{C_i=1}) or experimental arm  (\eqn{ C_i= \{2,\ldots, A\}}).
#' The random numbers are drawn under a Bayesian outcome-adaptive assignment model.
#'
#'
#' @param Accrual    The number of patients randomized to each treatment arm.
#' @param k.c        Current number of groups of added arms (initial group counts too).
#'
#' @param batch      Group membership for each treatment arm (for control put 1).
#' @param m.k        Mean parameter of the Gompertz function for each treatment group,
#' @param n.k        Extension of the overall sample size of the trial after each group of added arms.
#'
#' @param PrPTE      \code{PrPTE} is a vector which contains, for each experimental arm,
#'                   the posterior probability of treatment efficacy
#'                   \code{PrPTE[a]} \eqn{=p(\theta_a > \theta_0 | Data)}.
#'                   The vector can be generated with \code{\link{Posterior.with.control}}.
#'
#' @param rand       A vector three tuning parameter for adaptive randomization \code{rand =c(a,b,c)}.
#'                   The first and second element correspond to the maximum
#'                   and the shap parameter of the power function \eqn{h(x) = a * (x/n)^b}
#'                   The last parameter controls the randomization to the control.
#'
#' @param q.vec      The 3 common parameter \eqn{ (r_0, r_1,r_3) } of the Gompertz function
#'                   \eqn{q_k(x) = r_0 + r_1 \exp(-\exp( -r_2 [x-m_k] ))}.
#'
#' @param n.a.max    Vector of maximum sample sizes for each experimental arm.
#' @param Active     Activity status for each experimental arm at the i-th arrival,
#'                   i.e. \code{Active[k]=1} or \code{TRUE} if  k-th arm is active.
#'
#' @param nr.a       How many random numbers should be generated
#'
#' @return
#' Returns a set of \code{nr.a} treatment assignment numbers.
#'
#' @examples
#' # current outcome data
#' Outcome      = rbind( N=c(34, 32, 25, 0),
#'                       X=c(20, 19, 22, 0))
#'
#' # activity status and futility stopping parameter
#' Accrual      = c(44, 42, 35, 5)
#'
#' PrPTE        = Posterior.with.control(Outcome)
#'
#' # add 3th experimental arm after 101 patients
#' added.at     = c(0,0,0, 102)
#' batch        = c(1,1,1, 2)
#' rand         = c(H=2, gamma=1, b=.1)
#' q.vec        = c(r1=1, r2=2, r3=.1)
#'
#' # change platou of the Gompertz function after 60 and 30 Accrual patients
#' m.k          = c(60, 30)
#'
#' # extend sample size by 63 patients
#' n.k          = c(2*63, 63)
#' n.a.max      = rep(63, 3)
#'
#' Randomize.BAR.add(Accrual, k.c=2, batch, m.k, n.k, PrPTE, rand, q.vec, n.a.max, nr.a=5)
#' @export
Randomize.BAR.add = function(Accrual, k.c, batch, m.k, n.k, PrPTE, rand, q.vec, n.a.max, Active=rep(1,length(PrPTE)), nr.a=1){

  Accrual.to.batch =  sapply(1:k.c, function(k)  sum(Accrual[which(batch[-1]==k)+1]) )
  q.batch             =  q.vec[1] + q.vec[2] * exp(-exp(q.vec[3]*(Accrual.to.batch-m.k)))
  h.batch             =  rand[1] * (Accrual.to.batch/n.k)^rand[2]
  id.k                =  Accrual.to.batch>=n.k
  id.n                =  Accrual[-1] < n.a.max
  h.batch[id.k]       =  rand[1]

  h.a                 = h.batch[batch[-1]]
  q.a                 = q.batch[batch[-1]]

  # randomization probabilities for exparimetal arms
  id.active           = Active==1
  Pr.a                = PrPTE^h.a * q.a * id.n * id.active

  # randomization probability for control arm
  P0 = exp(rand[3] * (max(Accrual[-1])-Accrual[1])) / sum(id.n * id.active)

  # combine all randomization probabilities
  Pr = c(P0, Pr.a/sum(Pr.a) )
  Pr = Pr/sum(Pr)

  # avoide extreme unbalancing
  id.u = (Pr[-1] < 1/(3* sum(id.active & id.n) )) & id.active & id.n
  Pr[-1][id.u]= 1/(3* sum(id.active & id.n))

  return(sample.int(length(Pr), size=1, prob=Pr))}



#' Simulate a Trial using Bayesian-outcome adaptive (BOA) randomization
#'
#' Generate a multi-arm trial under BOA with early stopping for futility
#' and the possibility of adding arms to the trial.
#'
#' @param Potential.Outcome
#' A matrix with one potential outcome data for each arm.
#' The data can be generated with \code{\link{Generat.Outcome}}.
#'
#' @param added.at
#' A vector of adding times for each arm.
#' For all initial arms use \code{0}.
#'
#' @param Outcome.delay
#' Expected time between enrolments and avalability of the outcome.
#' I.e. if the endpoint is respons to treatment at 6 month, \code{Outcome.delay=6}.
#'
#' @param n.a.max    Vector of maximum sample sizes for each experimental arm.
#'
#' @param rand       A vector \code{rand =c(H, gamma, b)} of randomization three tuning parameter.
#'                   The first and second element correspond to the maximum
#'                   and the shap parameter of the power function \eqn{h(x) = H * (x/n)^gamma}
#'                   The last parameter controls the randomization to the control.
#'
#' @param q.vec      The 3 common parameter \eqn{ (r_0, r_1,r_3) } of the Gompertz function
#'                   \eqn{q_k(x) = r_0 + r_1 \exp(-\exp( -r_2 [x-m_k] ))}.
#'
#' @param m.k                Mean parameter of the Gompertz function for each batch
#' @param n.k                Extension of the overall sample size for each added group
#'
#' @param stop.par           Vector of stopping parameters with default c(f=0, g=1).
#' @param do.futi.stop       If \code{TRUE} do sequantial futility analysis.
#'
#' @return
#' The function returns a list \code{list(Potential.Outcome, Outcome=Outcome, Active=Active)} of three elements.
#'
#' \code{Potential.Outcome} is identical to the impute matrix \code{Potential.Outcome}
#' except that the 2nd  column now contains treatment assignment variables, i.e.
#' \code{Potential.Outcome[10,2]} equals one if the 10th patients was Accrual to the control.
#' \code{Potential.Outcome[10,2]} equals 2 if the 10th patients was Accrual to the arm 1.
#'
#' \code{Outcome} is an array of outcome data,
#' where \code{Outcome[,,k]} corresponds to the outcome data in the kth group of added arms.
#'
#' \code{Active} corresponds to the activity status of each arm.
#'
#' @examples
#' # generate outcome data
#' X = Generat.Outcome(N=5*63, p.vec=rep(.3, 5), A.rate=6)
#'
#' #generate trial
#' Simulate.BAR.add.trial(
#'   Potential.Outcome = X,
#'   # add one arm after 79 & another after 99 patients
#'   added.at          = c(0,0,0, 80,100),
#'   Outcome.delay     = 20,
#'
#'   # randomize at most 75 patients to each arm
#'   n.a.max           = rep(75, 4),
#'
#'   # randomization parameter
#'   rand              = c(a=1, b=1, c=.1),
#'   q.vec             = c(1,2, .1),
#'   m.k               =   c(60, 30, 30),
#'   n.k               = c(2*63, 63, 63),
#'
#'   # futility stopping parameter
#'   stop.par          = c(f=.3, gf=1),
#'   do.futi.stop      = TRUE )
#'
#' @export
Simulate.BAR.add.trial =
  function(Potential.Outcome, added.at=rep(1, ncol(Potential.Outcome)), Outcome.delay=0, n.a.max,
           rand, m.k, n.k, q.vec, stop.par=c(f=0, gf=1), do.futi.stop=TRUE){

    N.A            = dim(Potential.Outcome) - c(0,2)
    M.k            = unique(added.at)
    A.k            = sapply(M.k, function(M.k) sum(added.at[-1]==M.k))
    K              = length(M.k)
    batch          = sapply(added.at, function(a) which(M.k==a))
    Outcome        = array(0L, dim=c(2, N.A[2], K) )
    Active         = rbind(rep(1L,N.A[2]), rep(0L,N.A[2]))
    Accrual        = rep(0, N.A[2])
    n.last         = 0

##############  active accrual period #####################
for(i in 1:N.A[1]){
      ## update data during the active accrual period
      id             = added.at<=i
      k.c            = sum(M.k<=i)
      t.i            = Potential.Outcome[i,1]
      Update.Outcome = Update.Data(i, t.i, Potential.Outcome, Outcome.delay, Outcome, n.last, batch, N.target.0=N.A[1], M.k, during.accrual=TRUE)
      Outcome        = Update.Outcome$Outcome
      n.last         = Update.Outcome$n.last
      Outcome.c      = sapply(1:sum(id), function(a) Outcome[,a, batch[a]] )
      PrPTE          = Posterior.with.control(Outcome=Outcome.c)

      ## futility testing
      if(do.futi.stop)
        Active[,id]= Apply.stopping.rules(i=i, Active=Active[,id], PrPTE=PrPTE, stop.par=stop.par, Accrual=Accrual[id], n.a.max=n.a.max[id[-1]])

      ## randomize
      if( any(Active[1,id][-1]==1 & Accrual[id][-1] <n.a.max[id[-1]])  ){
        Potential.Outcome[i,2] = a.i = Randomize.BAR.add(Accrual=Accrual[id], k.c=k.c, batch=batch[id], m.k=m.k[1:k.c], n.k=n.k[1:k.c], PrPTE=PrPTE, rand=rand, q.vec=q.vec, n.a.max=n.a.max[id[-1]], Active = Active[1,id][-1] )
        Accrual[a.i]        = Accrual[a.i] + 1
      }
}

############## follow up: update data after  the active accrual period #####################
    if(n.last< N.A[1]){
      id     = rep(TRUE, N.A[2])

      for( j in (n.last+1):N.A[1]){
        if(any(Active[1,id][-1]==1)){
                                a                    = Potential.Outcome[j,2]
          if(a>1)               Outcome[,a,batch[a]] = Outcome[,a,batch[a]] + c(1L, Potential.Outcome[j,2+a])
          if(a==1){
            for(k in 1:sum(M.k <= i)) if(j>M.k[k])
                                Outcome[,1,k]        = Outcome[,1,k]        + c(1L, Potential.Outcome[j,3])  }

          # apply futility stopping rules
        if(do.futi.stop){
          Outcome.c   = sapply(1:sum(id), function(a) Outcome[,a, batch[a]] )
          PrPTE       = Posterior.with.control(Outcome=Outcome.c)
          Active[,id]= Apply.stopping.rules(i=i, Active=Active[,id], PrPTE=PrPTE, stop.par=stop.par, Accrual=Accrual[id], n.a.max=n.a.max[id[-1]])
        }
        }}}

    return(list(TrialData=Potential.Outcome, Outcome=Outcome, Active=Active)) }


