#' Random Numbers for Doubly Adaptive Biased Coin Designs
#'
#' Generate a vector of \code{nr.a} intergers of treatment assignments
#' under doubly adaptive biased coin designs.
#'
#' The function generates integers between from  \eqn{ \{1,\ldots, A\}}, where \code{A=length(PrPTE)-1}.
#' Each random integer \eqn{ C_i= \{1,\ldots, A\}} represents the treatment assignment to the control,
#' if  \eqn{C_i=1} or experimental arm  \eqn{ C_i= \{2,\ldots, A\}}.
#'
#' The random numbers are drawn under a doubly adaptive biased coin designs assignment model.
#'
#' @param Accrual    The number of patients Accrual to each treatment arm.
#'
#' @param Outcome    A matrix, where the 1./2. row contains the observed/positive outcomes for each active arm.
#' @param k.c        Current number of added groups (initial groups counts too)
#'
#' @param batch      Group membership for each treatment arm (for control put 1)
#'
#' @param h.k        Initial level of corrections for each batch
#' @param n.k        Extension of the overall sample size for goup of added arms
#'
#' @param rand       A vector of tuning parameter.
#' @param n.a.max    Vector of maximum sample sizes for each experimental arm.
#' @param Active     Activity status for each arm at the i-th arrival.
#'
#' @examples
#' # current outcome data
#' Outcome      = rbind( N=c(34, 32, 25, 0),
#'                       X=c(20, 19, 22, 0))
#'
#' # activity status and futility stopping parameter
#' Accrual      = c(44, 42, 35, 5)
#'
#' PrPTE        = Posterior.with.control(Outcome)
#'
#' # add 3th experimental arm after 101 patients
#' added.at     = c(0,0,0, 102)
#' batch        = c(1,1,1, 2)
#'
#' rand         = c(H=3, gamma=1)
#' h.k          = c(h1=1, h2=2)
#'
#' # extend sample size by 63 patients
#' n.k          = c(2*63, 63)
#' n.a.max      = rep(63, 3)
#'
#' Randomize.DBCD.add(Accrual, Outcome, k.c=2, batch, h.k, n.k, rand, n.a.max)
#' @export
Randomize.DBCD.add = function(Accrual, Outcome, k.c, batch, h.k, n.k, rand, n.a.max, Active=rep(1,length(Accrual)) ){

  # get hyper-parameter
  X         = Outcome[2,]
  N         = Outcome[1,]
  # v         = expand.grid(seq(1, 9, 1.5), seq(1, 9, 1.5))
  # BB        = function(v) sum(lbeta(X+v[1], N-X+v[2])) - length(X)*sum(lbeta(v[1], v[2]))
  # ML        = apply(v, 1, BB)
  v         = c(1,1) # as.double(v[which.max(ML),])

  # activity status
  id.a = Active[-1]==1
  id.n = Accrual[-1] < n.a.max

  # estimated response rate
  theta     = (v[1]+X)/(sum(v) +N)  *  c(TRUE, id.a & id.n)
  theta[1]  = max(theta[-1])

  # compute power-function
  Accrual.to.batch =  sapply(1:k.c, function(k)  sum(Accrual[which(batch[-1]==k)+1]) )
  h.batch             =  h.k + rand[1] * (Accrual.to.batch/n.k)^rand[2]
  id.k                =  Accrual.to.batch>=n.k
  h.batch[id.k]       =  rand[1]
  h.a                 =  h.batch[batch[-1]]

  # estimated target allocation
  p.a       = sqrt(theta) *  c(TRUE, id.a & id.n)
  p.a       = p.a/sum(p.a)

  # empirical allocation
  p.emp     = (Accrual+5) *  c(TRUE, id.a & id.n)
  p.emp     = p.emp/sum(p.emp)

  # corrections factor
  q.a       = (p.a/p.emp)^c(2, h.a);    q.a[!c(TRUE, id.a & id.n)] = 0

  ## power function
  Pr    = p.a * q.a
  Pr    = Pr/sum(Pr)

  # avoide extreme unbalancing
  id.u      = (Pr < 1/(3* sum(c(TRUE, id.a & id.n)) )) &  c(TRUE, id.a & id.n)
  Pr[id.u]  = 1/(3* sum(sum(c(TRUE, id.a & id.n)) ))


  return(sample.int(length(N), size=1, prob=Pr)) }













#' Simulate a Trial under Doubly Adaptive Biased Coin Designs with Futility Stopping
#'
#' Generate a multi-arm trial under a Doubly Adaptive Biased Coin designs
#' with early stopping for futility and the possibility of adding one or multiple arms to the trial.
#'
#' @param Potential.Outcome
#' A matrix with one potential outcome data for each arm.
#' The data can be generated with \code{\link{Generat.Outcome}}.
#'
#' @param added.at
#' A vector of adding times for each arm.
#' For all initial arms use \code{0}.
#'
#' @param Outcome.delay
#' Expected time between enrolments and avalability of the outcome.
#' I.e. if the endpoint is respons to treatment at 6 month, \code{Outcome.delay=6}.
#'
#' @param rand       A vector \code{rand =c(H, gamma)} of randomization three tuning parameter.
#'                   The first and second element correspond to the maximum
#'                   and the shap parameter of the power function \eqn{h(x) = H * (x/n)^gamma}
#'                   The last parameter controls the randomization to the control.#'
#'
#' @param h.k                Initial level of corrections for each group of added arms.
#' @param n.k                Extension of the overall sample size for each group of added arms.
#' @param n.a.max            Vector of maximum sample sizes for each experimental arm.
#'
#' @param stop.par           Vector of stopping parameters with default c(f=0, gf=1).
#' @param do.futi.stop       If \code{TRUE} do sequantial futility analysis.
#'
#' @return
#' The function returns a list \code{list(Potential.Outcome, Outcome=Outcome, Active=Active)} of three elements.
#'
#' \code{Potential.Outcome} is identical to the impute matrix \code{Potential.Outcome}
#' except that the 2nd  column now contains treatment assignment variables, i.e.
#' \code{Potential.Outcome[10,2]} equals one if the 10th patients was Accrual to the control.
#' \code{Potential.Outcome[10,2]} equals 2 if the 10th patients was Accrual to the arm 1.
#'
#' \code{Outcome} is an array of outcome data,
#' where \code{Outcome[,,k]} corresponds to the outcome data in the kth group of added arms.
#'
#' \code{Active} corresponds to the activity status of each arm.
#'
#' @examples
#' # generate outcome data
#' X = Generat.Outcome(N=5*63, p.vec=rep(.3, 5), A.rate=6)
#'
#' # generate trial
#' Simulate.DBCD.add.trial(
#'   Potential.Outcome = X,
#'
#'   # add one arm after 79 & another after 99 patients
#'   added.at          = c(0,0,0, 80,100),
#'   Outcome.delay     = 20,
#'
#'   # randomize at most 75 patients to each arm
#'   n.a.max           = rep(75, 4),
#'
#'   # randomization parameter
#'   rand              = c(a=1, b=1, c=.1),
#'   h.k               = c(0, 1, 1),
#'   n.k               = c(2*63, 63, 63),
#'
#'   # futility stopping parameter
#'   stop.par          = c(f=.3, gf=1),
#'   do.futi.stop      = TRUE )
#'
#' @export
Simulate.DBCD.add.trial =
  function(Potential.Outcome, added.at=rep(0, ncol(Potential.Outcome)), Outcome.delay, n.a.max,
           rand, h.k, n.k, stop.par=c(f=0, gf=1), do.futi.stop=TRUE){

  N.A            = dim(Potential.Outcome) - c(0,2)
  M.k            = unique(added.at)
  A.k            = sapply(M.k, function(M.k) sum(added.at[-1]==M.k))
  K              = length(M.k)
  batch          = sapply(added.at, function(a) which(M.k==a))
  Outcome        = array(0L, dim=c(2, N.A[2], K) )
  Active         = rbind(rep(1L,N.A[2]), rep(0L,N.A[2]))
  Accrual     = rep(0, N.A[2])
  n.last         = 0

##############  active accrual period #####################
for(i in 1:N.A[1]){
    ## update data during the active accrual period
    id             = added.at<=i
    k.c            = sum(M.k<=i)
    t.i            = Potential.Outcome[i,1]
    Update.Outcome = Update.Data(i, t.i, Potential.Outcome, Outcome.delay, Outcome, n.last, batch, N.target.0=N.A[1], M.k, during.accrual=TRUE)
    Outcome        = Update.Outcome$Outcome
    n.last         = Update.Outcome$n.last
    Outcome.c      = sapply(1:sum(id), function(a) Outcome[,a, batch[a]] )

    ## futility testing
    if(do.futi.stop){
     PrPTE       = Posterior.with.control(Outcome=Outcome.c)
     Active[,id] = Apply.stopping.rules(i=i, Active=Active[,id], PrPTE=PrPTE, stop.par=stop.par, Accrual=Accrual[id], n.a.max=n.a.max[id[-1]])
    }

    ## randomize
    if( any(Active[1,id][-1]==1 & Accrual[id][-1] <n.a.max[id[-1]])  ){
      Potential.Outcome[i,2] = a.i =  Randomize.DBCD.add(Accrual=Accrual[id], Outcome=Outcome.c, k.c=k.c, batch=batch[id], h.k=h.k[1:k.c], n.k=n.k[1:k.c], rand=rand, n.a.max=n.a.max[id[-1]])
      Accrual[a.i]        = Accrual[a.i] + 1
    }
}

############## follow up: update data after  the active accrual period #####################
  if(n.last< N.A[1]){
    id     = rep(TRUE, N.A[2])

    for( j in (n.last+1):N.A[1]){
      if(any(Active[1,id][-1]==1)){
                             a                    = Potential.Outcome[j,2]
        if(a>1)              Outcome[,a,batch[a]] = Outcome[,a,batch[a]] + c(1L, Potential.Outcome[j,2+a])
        if(a==1){
          for(k in 1:sum(M.k <= i)) if(j>M.k[k])
                             Outcome[,1,k]        = Outcome[,1,k]        + c(1L, Potential.Outcome[j,3])  }

      # apply futility stopping rules
      if(do.futi.stop){
        Outcome.c = sapply(1:sum(id), function(a) Outcome[,a, batch[a]] )
        PrPTE     = Posterior.with.control(Outcome=Outcome.c)
        Active    = Apply.stopping.rules(i=i, Active=Active[,id], PrPTE=PrPTE, stop.par=stop.par, Accrual=Accrual[id], n.a.max=n.a.max[id[-1]])
      }
      }}}

  return(list(TrialData=Potential.Outcome, Outcome=Outcome, Active=Active)) }





