#' Generate Outcome Data
#'
#' Generate arrival times and potentrial outcomes
#'
#' @param N       Generate outcomes and arrival times for \code{N} observations.
#' @param p.vec   A vector of probabilities of a positive response to each treatment.
#' @param A.rate  Mean number of patients randomized to treatment within a given unit of time, typically month.
#'
#' @details
#' The function generates arrival times for  \code{N} patients according to a Poisson process.
#' Moreover for each patient one response outcome is generated
#' under each treatment response probability \code{p.vec}.
#'
#' @return
#' The function returns a matrix with one row for each patient.
#' The first element of the row denotes the arrival time of the patient.
#' The remaining elements of the row k=2, ...,  denote the outcome of the patient if she/he
#' would have been treated with dose k.
#'
#' @export
#'
#' @examples
#' # response rate under the control,  treatment 1 & 2
#' p.vec  = c(.1, .5, .9)
#' N      = 10            # 10 patients
#' A.rate = 6             # Accrual rate of 6 patients per month
#'
#' Generat.Outcome(N=N, p.vec=p.vec, A.rate=A.rate)
#'
Generat.Outcome = function(N, p.vec, A.rate=1)
   cbind(AT=cumsum(rexp(n = N, rate = A.rate)), a=rep(0, N), X=sapply(p.vec, function(p) rbinom(N, 1, p)))

